#include <iostream>
#include <fstream>
#include <string.h>
#include <stdlib.h>
using namespace std;

const int MAXS = 100;
const int MAXT = 50;
const int MAXP = 80;

const float COSTO_KM_REG = 0.1f;
const float COSTO_KM_IC = 0.2f;
const float COSTO_KM_FRECCIA = 0.3f;

enum Tipo {REG, IC, FRECCIA};

struct Orario {
  int minuti;
  int ora;
};

struct Segmento {
  char stazionePart[MAXS];
  Orario oraPart;
  char stazioneArr[MAXS];
  Orario oraArr;
  float km;
};

struct Treno {
  int id;
  Tipo tipo;
  Segmento segm[MAXT];
  int numSegm;
};

struct Programmazione {
  Treno tre[MAXP];
  int numTre;
};

float costoBiglietto(const Programmazione* pp, int id); 
int partenza(const Programmazione* pp, const char staz[], Orario inizio, Orario fine, int vett[]);
bool salva(const Programmazione* pp, const char nomef[], const char p[], const char a[]);
void stampaOrd(const Programmazione* pp);
Tipo tipoPiuNumeroso(const Programmazione* pp);

void menu(){
  cout << "-----------------------" << endl;
  cout << "1) costo biglietto" << endl;
  cout << "2) partenza" << endl;
  cout << "3) salva" << endl;
  cout << "4) stampa ord" << endl;
  cout << "5) tipo piu numeroso" << endl;
  cout << "6) stampa dati treni" << endl;
  cout << "7) esci" << endl;
  cout << "? ";
}

Orario leggiOrario(){
  Orario o;
  cout << "Ora? "; 
  cin >> o.ora;
  cout << "Minuti? ";
  cin >> o.minuti;
  return o;
}

int carica(Programmazione* pp, const char fn[]) {
  fstream f;
  f.open(fn, ios::in);
  if(!f) return false;
  char c;
  Treno t;
  int q = 0;
  while(true) {
    f >> t.id;
    if(!f) break;
    f >> c; 
    switch(c) {
      case 'R': t.tipo = REG; break;
      case 'I': t.tipo = IC; break;
      case 'F': t.tipo = FRECCIA; break;
    }
    f >> t.numSegm;
    for(int i=0; i<t.numSegm; i++) {
      f >> t.segm[i].stazionePart;
      f >> t.segm[i].oraPart.ora;
      f >> c;  
      f >> t.segm[i].oraPart.minuti;
      f >> t.segm[i].stazioneArr;
      f >> t.segm[i].oraArr.ora;
      f >> c;  
      f >> t.segm[i].oraArr.minuti;
      f >> t.segm[i].km;
    }
    pp->tre[q++] = t;
  }
  pp->numTre = q;
  return q;
}

void stampaTipo(Tipo t) {
  switch(t) {
    case REG: cout << "REG"; break;
    case IC: cout << "IC"; break;
    case FRECCIA: cout << "FRECCIA"; break;
  }
}

void stampaOrario(Orario o) {
  cout << o.ora << ':' << o.minuti;
} 

void stampaTreno(const Treno* pt) {
  cout << pt->id << ' ';
  stampaTipo(pt->tipo);
  cout << ' ' << pt->numSegm << endl;
  for(int i=0; i<pt->numSegm; i++) {
    cout << pt->segm[i].stazionePart << ' '; 
    stampaOrario(pt->segm[i].oraPart);
    cout << ' ' << pt->segm[i].stazioneArr << ' ';
    stampaOrario(pt->segm[i].oraArr);
    cout << endl;
  }
  cout << endl;
}

void stampaTutto(const Programmazione* pp) {
  for(int i=0; i<pp->numTre; i++)
    stampaTreno(&pp->tre[i]);
}

int main(){
  
  char s;
  float costo;
  int id;
  char staz1[MAXS];
  char staz2[MAXS];
  Orario o1, o2;
  int n;
  int vett[MAXT];
  Tipo t;
  bool res;

  Programmazione prog;
  n = carica(&prog, "input.txt");
  cout << "Ho caricato i dati di " << n << " treni" << endl;

  while(true) {
    menu();
    cin >> s;
    switch(s) {
      case '1': 
        cout << "Inserisci id: ";
        cin >> id;
        costo = costoBiglietto(&prog, id);
	cout << "Costo: " << costo << endl;
        break;
      case '2':
	cout << "Stazione? ";
        cin >> staz1;
        cout << "Orario inizio" << endl;
        o1 = leggiOrario();
        cout << "Orario fine" << endl;
        o2 = leggiOrario();
        n = partenza(&prog, staz1, o1, o2, vett);
        cout << "Ci sono i seguenti treni:" << endl;
        for(int i=0; i<n; i++)
          cout << vett[i] << endl;
        break;
      case '3':
        cout << "Stazione di partenza? ";
        cin >> staz1;
        cout << "Stazione di arrivo? ";
        cin >> staz2;
        res = salva(&prog, "out.txt", staz1, staz2);
        if(res) {
          cout << "Elenco:" << endl;
          system("type out.txt"); 
        } else cout << "Problema nel salvataggio" << endl;
        break;
      case '4':
        stampaOrd(&prog);
        break;
      case '5':
        t = tipoPiuNumeroso(&prog);
        cout << "Il tipo piu' numeroso e': ";
        switch(t) {
          case REG: cout << "REG"; break;
          case IC: cout << "IC"; break;
          case FRECCIA: cout << "FRECCIA"; break;
        }
        cout << endl;
        break;
      case '6':
        stampaTutto(&prog);
        break;
      case '7':
        return 0;
    }
  }  
}
