#include <iostream>
#include <fstream>
#include <string.h>
using namespace std;

const int MAXS = 100;
const int MAXT = 50;
const int MAXP = 80;

const float COSTO_KM_REG = 0.1f;
const float COSTO_KM_IC = 0.2f;
const float COSTO_KM_FRECCIA = 0.3f;

enum Tipo {REG, IC, FRECCIA};

struct Orario {
  int minuti;
  int ora;
};

struct Segmento {
  char stazionePart[MAXS];
  Orario oraPart;
  char stazioneArr[MAXS];
  Orario oraArr;
  float km;
};

struct Treno {
  int id;
  Tipo tipo;
  Segmento segm[MAXT];
  int numSegm;
};

struct Programmazione {
  Treno tre[MAXP];
  int numTre;
};

float costoKm(Tipo t) {
  switch(t) {
    case REG: return COSTO_KM_REG;
    case IC: return COSTO_KM_IC;
    case FRECCIA: return COSTO_KM_FRECCIA;
  }
}

float costoBiglietto(const Programmazione* pp, int id) {
  for(int i=0; i<pp->numTre; i++)
    if(pp->tre[i].id == id) {
      float dist = 0.0f;
      for(int j=0; j<pp->tre[i].numSegm; j++) 
        dist += pp->tre[i].segm[j].km;
      return dist*costoKm(pp->tre[i].tipo); 
    }
  return 0.0f;
}

int confronta(Orario x, Orario y) {
  if(x.ora != y.ora) return x.ora - y.ora;
  return x.minuti - y.minuti;
}

bool compreso(Orario o, Orario a, Orario b) {
  return confronta(a, o) <= 0 && confronta(o, b) <= 0;  
}

bool controllaParte(const Treno* pt, const char staz[], Orario inizio, Orario fine) {
  for(int i=0; i<pt->numSegm; i++)
    if(strcmp(pt->segm[i].stazionePart, staz) == 0 && compreso(pt->segm[i].oraPart, inizio, fine))
      return true;
  return false;
}

int partenza(const Programmazione* pp, const char staz[], Orario inizio, Orario fine, int vett[]) {
  int quanti = 0;
  for(int i=0; i<pp->numTre; i++)
    if(controllaParte(&pp->tre[i], staz, inizio, fine)) {
      vett[quanti] = pp->tre[i].id;
      quanti++;
    }
  return quanti;
}

bool controllaPA(const Treno* pt, const char p[], const char a[], Orario* po1, Orario* po2) {
  bool flag = false;
  for(int i=0; i<pt->numSegm; i++) {
    if(strcmp(pt->segm[i].stazionePart, p) == 0) {
      flag = true;
      *po1 = pt->segm[i].oraPart;
    }
    if(flag && strcmp(pt->segm[i].stazioneArr, a) == 0) {
      *po2 = pt->segm[i].oraArr;
      return true;
    }
  }
  return false;
}

bool salva(const Programmazione* pp, const char nomef[], const char p[], const char a[]) {
  fstream f;
  f.open(nomef, ios::out);
  if(!f)
    return false;
  Orario o1, o2;
  for(int i=0; i<pp->numTre; i++)
    if(controllaPA(&pp->tre[i], p, a, &o1, &o2)) {
      f << pp->tre[i].id << ' ';
      switch(pp->tre[i].tipo) {
        case REG: f << "REG "; break;
        case IC: f << "IC "; break;
        case FRECCIA: f << "FRECCIA "; break;
      }	
      f  << p << ' ' <<  o1.ora << ':' << o1.minuti << ' '
         << a << ' ' << o2.ora << ':' << o2.minuti << endl;
    }
  f.close();
  return true;
}

struct Aux {
  int id;
  char orig[MAXS];
};


void ordina(Aux v[], int n) {
  for(int i=0; i<n-1; i++) {
    int index = i;
    for(int j=i+1; j<n; j++)
      if(strcmp(v[j].orig, v[index].orig) < 0)
        index = j;
    Aux a = v[i];
    v[i] = v[index];
    v[index] = a;
  }
}

void stampaOrd(const Programmazione* pp) {
  Aux aux[MAXP];
  int q = 0;
  for(int i=0; i<pp->numTre; i++) {
    aux[q].id = pp->tre[i].id;
    strcpy(aux[q].orig, pp->tre[i].segm[0].stazionePart);
    q++;	
  }
  ordina(aux, q);
  for(int i=0; i<q; i++)
    cout << aux[i].id << ' ' << aux[i].orig << endl;
}

Tipo tipoPiuNumeroso(const Programmazione* pp) {
  int v[3] = {0};
  for(int i=0; i<pp->numTre; i++)
    v[pp->tre[i].tipo]++;
  if(v[0] > v[1]) 
    if(v[0] > v[2]) return REG;
    else return FRECCIA;
  else
    if(v[1] > v[2]) return IC;
    else return FRECCIA;
}

